@@ -849,10 +849,15 @@ extract_metadata (MetadataExtractor      *extractor,
                   const gchar            *file_url)
 {
 	TrackerResource *resource;
+	gchar *resource_uri;
+	GFile *file;
 
 	g_return_val_if_fail (extractor != NULL, NULL);
 
-	resource = tracker_resource_new (NULL);
+	file = g_file_new_for_uri (file_url);
+	resource_uri = tracker_file_get_content_identifier (file, NULL, "1");
+	resource = tracker_resource_new (resource_uri);
+	g_free (resource_uri);
 
 	if (extractor->toc) {
 		gst_tag_list_insert (extractor->tagcache,
@@ -974,10 +979,18 @@ extract_metadata (MetadataExtractor      *extractor,
 					/* Reuse the "root" InformationElement resource for the first track,
 					 * so there's no spare ones.
 					 */
-					if (node == extractor->toc->entry_list)
+					if (node == extractor->toc->entry_list) {
 						track = resource;
-					else
-						track = tracker_resource_new (NULL);
+					} else {
+						gchar *suffix, *resource_uri;
+
+						suffix = g_strdup_printf ("%d", g_list_position (extractor->toc->entry_list,
+						                                                 node) + 1);
+						resource_uri = tracker_file_get_content_identifier (file, NULL, suffix);
+						track = tracker_resource_new (resource_uri);
+						g_free (resource_uri);
+						g_free (suffix);
+					}
 
 					extract_track (track, extractor, node->data, file_url, album_disc);
 					tracker_resource_set_relation (track, "nie:isStoredAs", file_resource);
@@ -1034,6 +1047,8 @@ extract_metadata (MetadataExtractor      *extractor,
 
 	common_extract_stream_metadata (extractor, file_url, resource);
 
+	g_object_unref (file);
+
 	return resource;
 }
 
@@ -1229,7 +1244,12 @@ discoverer_init_and_run (MetadataExtractor *extractor,
 	extractor->duration = gst_discoverer_info_get_duration (info) / GST_SECOND;
 
 	/* Retrieve global tags */
+#if defined(HAVE_GSTREAMER_1_20)
+	GstDiscovererStreamInfo *sinfo = gst_discoverer_info_get_stream_info (info);
+	discoverer_tags = gst_discoverer_container_info_get_tags ((GstDiscovererContainerInfo *)sinfo);
+#else
 	discoverer_tags = gst_discoverer_info_get_tags (info);
+#endif
 
 	if (discoverer_tags) {
 		gst_tag_list_insert (extractor->tagcache,
@@ -1241,7 +1261,6 @@ discoverer_init_and_run (MetadataExtractor *extractor,
 	extractor->streams = gst_discoverer_info_get_stream_list (info);
 	for (l = extractor->streams; l; l = g_list_next (l)) {
 		GstDiscovererStreamInfo *stream = l->data;
-		const GstTagList *stream_tags;
 
 		if (G_TYPE_CHECK_INSTANCE_TYPE (stream, GST_TYPE_DISCOVERER_AUDIO_INFO)) {
 			GstDiscovererAudioInfo *audio = (GstDiscovererAudioInfo*)stream;
@@ -1270,17 +1289,30 @@ discoverer_init_and_run (MetadataExtractor *extractor,
 		} else {
 			/* Unknown type - do nothing */
 		}
+	}
 
-		stream_tags = gst_discoverer_stream_info_get_tags (stream);
+	for (l = extractor->streams; l; l = g_list_next (l)) {
+		GstDiscovererStreamInfo *stream = l->data;
+		GstTagList *stream_tags;
+
+		stream_tags = gst_tag_list_copy (gst_discoverer_stream_info_get_tags (stream));
+		if (extractor->has_video &&
+		    gst_tag_list_get_tag_size (extractor->tagcache, "title") > 0)
+			gst_tag_list_remove_tag (stream_tags, "title");
 
 		if (stream_tags) {
 			gst_tag_list_insert (extractor->tagcache,
 					     stream_tags,
 					     GST_TAG_MERGE_APPEND);
 		}
+
+		gst_tag_list_unref (stream_tags);
 	}
 
 	gst_discoverer_info_unref (info);
+#if defined(HAVE_GSTREAMER_1_20)
+	gst_discoverer_stream_info_unref (sinfo);
+#endif
 
 	return TRUE;
 }
@@ -1413,16 +1445,7 @@ tracker_extract_module_init (GError **error)
 		"fluiddec",
 		"vaapi",
 		"video4linux2",
-		"nvmpegvideodec",
-		"nvmpeg2videodec",
-		"nvmpeg4videodec",
-		"nvh264sldec",
-		"nvh264dec",
-		"nvjpegdec",
-		"nvh265sldec",
-		"nvh265dec",
-		"nvvp8dec",
-		"nvvp9dec",
+		"nvcodec",
 	};
 	GstRegistry *registry;
 	guint i;
@@ -1434,8 +1457,11 @@ tracker_extract_module_init (GError **error)
 		GstPlugin *plugin =
 			gst_registry_find_plugin (registry,
 						  blocklisted[i]);
-		if (plugin)
+		if (plugin) {
+			g_debug ("Removing GStreamer plugin '%s' from registry",
+			         blocklisted[i]);
 			gst_registry_remove_plugin (registry, plugin);
+		}
 	}
 
 	return TRUE;
